Object.defineProperty(exports, "__esModule", {
  value: true
});

var _templateObject = _taggedTemplateLiteral(['', ''], ['', '']);

var _path = require('path');

var _os = require('os');

var _fs = require('fs');

var _babylon = require('babylon');

var _graphqlTag = require('graphql-tag');

var _graphqlTag2 = _interopRequireDefault(_graphqlTag);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _taggedTemplateLiteral(strings, raw) { return Object.freeze(Object.defineProperties(strings, { raw: { value: Object.freeze(raw) } })); }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.default = function (_ref) {
  var t = _ref.types;
  return {
    visitor: {
      ImportDeclaration: {
        exit: function exit(curPath, state) {
          var importPath = curPath.node.source.value;
          if (importPath.endsWith('.graphql') || importPath.endsWith('.gql')) {
            var query = createQuery(importPath, state.file.opts.filename);
            query.processFragments();
            query.dedupeFragments();
            query.parse();
            query.makeSourceEnumerable();
            curPath.replaceWith(t.variableDeclaration('const', [buildInlineVariable(query.ast)]));
          }

          function buildInlineVariable(graphqlAST) {
            var babelAST = (0, _babylon.parse)('const obj = ' + JSON.stringify(graphqlAST));
            var objExp = babelAST.program.body[0].declarations[0].init;
            return t.variableDeclarator(t.identifier(curPath.node.specifiers[0].local.name), t.objectExpression(objExp.properties));
          }
        }
      }
    }
  };
};

function createQuery(queryPath, babelPath) {
  var absPath = (0, _path.resolve)((0, _path.dirname)(babelPath), queryPath);
  var source = (0, _fs.readFileSync)(absPath).toString();
  var ast = null;
  var fragmentDefs = [];

  return {
    processFragments: function processFragments() {
      processImports(getImportStatements(source), (0, _path.dirname)(absPath));

      function getImportStatements(src) {
        return src.split(_os.EOL).filter(function (line) {
          return line.startsWith('#import');
        });
      }

      function processImports(imports, relDir) {
        imports.forEach(function (statement) {
          var fragmentPath = statement.split(' ')[1].slice(1, -1);
          var absFragmentPath = (0, _path.resolve)(relDir, fragmentPath);
          var fragmentSource = (0, _fs.readFileSync)(absFragmentPath).toString();
          var subFragments = getImportStatements(fragmentSource);
          if (subFragments.length > 0) {
            processImports(subFragments, (0, _path.dirname)(absFragmentPath));
          }
          fragmentDefs = [].concat(_toConsumableArray((0, _graphqlTag2.default)(_templateObject, fragmentSource).definitions), _toConsumableArray(fragmentDefs));
        });
      }
    },
    dedupeFragments: function dedupeFragments() {
      var seenNames = {};
      fragmentDefs = fragmentDefs.filter(function (def) {
        if (def.kind !== 'FragmentDefinition') return true;
        return seenNames[def.name.value] ? false : seenNames[def.name.value] = true;
      });
    },
    parse: function parse() {
      var parsedAST = (0, _graphqlTag2.default)(_templateObject, source);
      parsedAST.definitions = [].concat(_toConsumableArray(parsedAST.definitions), _toConsumableArray(fragmentDefs));
      ast = parsedAST;
    },
    makeSourceEnumerable: function makeSourceEnumerable() {
      var newAST = JSON.parse(JSON.stringify(ast));
      newAST.loc.source = ast.loc.source;
      ast = newAST;
    },

    get ast() {
      return ast;
    }
  };
}